function [w]=cauchy_fft(u,v,N)
%%% Inputs: u,v are vectors of lenght N*(N+1)/2
%%% Output: w is the cauchy product of u and v
%%% This function is faster than cauchy.m but gives larger error for large
%%% N.

%%% Step 1 : construction of the matrices corresponding to u and v, padded
%%% with zeros
Mu=zeros(N,N);
Mv=zeros(N,N);
for i=0:N-1
    Mu(1:N-i,i+1)=find_column(u,i,N-i-1);
    Mv(1:N-i,i+1)=find_column(v,i,N-i-1);
end

Mu_padded=zeros(4*N-1,4*N-1);
Mv_padded=zeros(4*N-1,4*N-1);
Mu_padded(2*N:3*N-1,2*N:3*N-1)=Mu;
Mv_padded(2*N:3*N-1,2*N:3*N-1)=Mv;

%%% Step 2 : computing the product with fft
itu=ifft2(ifftshift(Mu_padded));
itv=ifft2(ifftshift(Mv_padded));

itw=itu.*itv;
Mw=fftshift(fft2(itw));

%%%Step 3 : taking back the "interesting" part
Mw=(4*N-1)^2*Mw(2*N:3*N-1,2*N:3*N-1);
w=0*u;
for k=0:N-1    %k=|alpha|
    for j=0:k    %alpha_1=k-j,  alpha_2=j
        pos=position(k-j,j);
        w(pos)=Mw(j+1,k-j+1);
    end
end
