%% THIS SCRIPT COMPUTES AND VALIDATES A PARAMETERIZATION OF THE STABLE 
%% MANIFOLD OF THE ORIGIN. THIS PARAMETERIZATION IS OBTAINED WITH THE 
%% LARGEST POSSIBLE RESCALING gamma OF THE EIGENVECTORS THAT STILL GIVES AN
%% ERROR BOUND BELOW eps_max. WE DO THE COMPUTATION ANF PROOF FOR DIFFERENT
%% VALUES OF beta.


%% PARAMETERS OF THE EQUATION
Tab_beta=0.5:0.1:1.9;

%% PARAMETER FOR THE COMPUTATION
N=30;
N_bis=10;
nu=1;
r_max=10^-5;


%% COMPUTATION AND PROOF
p=zeros(4,1);
Tab_gamma=[];
Tab_a=[];
for beta=Tab_beta
    display(beta)
    lambda_r=-sqrt(2-beta)/2;
    lambda_i=sqrt(2+beta)/2;
    lambda=lambda_r+1i*lambda_i;
    V1=[1;lambda;lambda^2;lambda^3];
    V2=conj(V1);
    
    a_bis=zeros(4*N_bis*(N_bis+1)/2,1);
    a_bis(1:4)=p;
    a_bis(5:8)=V1;
    a_bis(9:12)=V2;
    test=norm_multiD(F(a_bis,N_bis,beta,lambda,p,V1,V2),N_bis,nu,4);

    % Before trying Newton's method, we now ensure that the scale is small enough,
    % if not we rescale the eigenvectors. If one of the Newton's method still does
    % not converge, decrease scale_para.
    gamma=1;
    scale_para=0.1;
    while test>scale_para
        gamma=gamma/2;
        V1=V1/2;
        V2=V2/2;
        a_bis(5:8)=V1;
        a_bis(9:12)=V2;
        test=norm_multiD(F(a_bis,N_bis,beta,lambda,p,V1,V2),N_bis,nu,4);
    end

    gamma_0=gamma;
    display(['First rescaling, gamma_0 = ',num2str(gamma_0)])

    % Newton's method
    tol=10^-14;
    nb_it=100;
    a_bis=Newton(a_bis,N_bis,beta,lambda,p,V1,V2,tol,nb_it,nu);

    % Construction of the full a
    a=zeros(4*N*(N+1)/2,1);
    a(1:4*N_bis*(N_bis+1)/2)=a_bis;

    % Computation of a precise numerical solution with Newton
    tol=10^-20;
    nb_it=5;
    a=Newton(a,N,beta,lambda,p,V1,V2,tol,nb_it,nu);
    
    % Getting the maximal gamma for which the proof works with r_0<r_max
    [a,gamma]=proof_max_r(a,N,beta,lambda,nu,r_max,p,V1,V2);
    
    display(gamma)
    
    Tab_a=[Tab_a a];
    Tab_gamma=[Tab_gamma gamma_0*gamma];
end

figure
plot(Tab_beta,Tab_gamma,'*k');

%% DISPLAY OF THE MANIFOLDS
%Since the phase space has dimension 4, we need to chose a projection
%before plotting. Take direction=[i,j,k] with i,j,k=1,...,4.

direction=[2 3 4];
num_angles=30;
num_rad = 10;
test_real=0;

figure
col=[0 0 1];

for k=1:length(Tab_beta)
    a=Tab_a(:,k);
    [TRI, W] = triangulate_complex(a,N,nu,num_angles,num_rad,direction,test_real);
    hold on
    t0 = trisurf(TRI, W(:,1),  W(:,2), W(:,3));
    set(t0, 'FaceColor', col);
    col=col +1/15*[1 1 -1];
end

