function [TRI, W] = triangulate_complex(a,N,nu,num_angles,num_rad,direction,test_real)

%==========================================================================
%Construction of the mesh:
%==========================================================================

theta_points = linspace( 0, 2*pi, num_angles);
r_points = linspace(0, nu, num_rad);

m_int = num_angles; 
n_int = num_rad; 

totalPossiblePoints = m_int*n_int;
D_parms = zeros(totalPossiblePoints, 2);

thisIndex = 1;

for i = 1:m_int
    for j = 1:n_int
        this_r = r_points(j);
        this_theta = theta_points(i);
        D_parms(thisIndex, 1:2) = this_r*[cos(this_theta), sin(this_theta)];    
        thisIndex = thisIndex + 1;
    end
end    

D_parms = unique(D_parms, 'rows');

%==========================================================================
%Triangulation of the parameter domain:
%==========================================================================

x = D_parms(:,1);
y = D_parms(:,2);

TRI = delaunay(x, y);

%==========================================================================
%Evaluation of the paramterization:
%==========================================================================

numPoints = size(D_parms, 1);
W = zeros(numPoints, 3);
[M1,M2,M3]=vect2mat(a,N,direction);

for k = 1:numPoints
    W(k, 1:3) = eval_parameterization(...
        D_parms(k, 1)+1i*D_parms(k, 2), ...
         D_parms(k, 1)-1i*D_parms(k, 2),M1,M2,M3,N,test_real)';
end