function [a,gamma_1,gamma_2]=compute_para_fast_slow(sigma,s,Delta,p,zeta,epsilon,N,nu,eps_max)
%%% This function compute an approximate parameterization of degre N of 
%%% the stable mnifold at p0 (with parameters sigma, s, Delta, p, zeta and 
%%% epsilon). This parameterization maximize the surface area 
%%% under the constrain of having a fixed ration (given by the inverse of 
%%% the ration of the eigenvalues) between the length of the
%%% eigenvectors, stil while having a defect being less than eps_max. 
%%% Outputs: the parameterization is gaiven by a vector a. gamma_1 and
%%% gamma_2 are the rescaling applied to the first and second eigenvector 
%%% respectively.


%% COMPUTATION OF THE FIXED POINT p AND OF THE ASSOCIATED STABLE EIGENVALUES AND EIGENVECTORS
g_u=@(u) u.^3-(1+sigma)*u.^2+(sigma+1/zeta)*u-p;
Dg=@(X) [0 1 0; 1/Delta*(3*X(1).^2-2*(1+sigma)*X(1)+sigma) s/Delta 1/Delta; epsilon/s 0 -zeta*epsilon/Delta];

u=fzero(g_u,0);

p0=[u;0;u/zeta];

[V,D]=eig(Dg(p0));

nb=0;
k=1;
Lambda=zeros(2,1);
VP=zeros(3,2);
while nb<2 && k<4
    if real(D(k,k))<0
        Lambda(nb+1)=D(k,k);
        VP(:,nb+1)=V(:,k);
        nb=nb+1;
    end
    k=k+1;
end

lambda_1=Lambda(1);
lambda_2=Lambda(2);
V1=VP(:,1);
V2=VP(:,2);

test_real=isreal([lambda_1,lambda_2]);
if test_real==0
    display('The eigenvalues are not real for the chosen set of parameters')
end

format long

display(p0)
display(lambda_1)
display(lambda_2)
display(V1)
display(V2)

ratio=abs(lambda_1/lambda_2);
display(ratio)

%% FIRST EASY NEWTON COMPUTATION WITH A FEW MODES
N_bis=10;
a_bis=zeros(3*N_bis*(N_bis+1)/2,1);
a_bis(1:3)=p;
a_bis(4:6)=V1;
a_bis(7:9)=ratio*V2;
error=norm_multiD(F(a_bis,N_bis,sigma,s,Delta,zeta,epsilon,lambda_1,lambda_2),N_bis,nu,3);

% Before trying Newton's method, we now ensure that the scale is small enough,
% if not we rescale the eigenvectors. If one of the Newton's method still does
% not converge, decrease scale_para.
gamma=1;
scale_para=0.1;
while error>scale_para
    gamma=gamma/2;
    a_bis(4:6)=gamma*V1;
    a_bis(7:9)=gamma*ratio*V2;
    error=norm_multiD(F(a_bis,N_bis,sigma,s,Delta,zeta,epsilon,lambda_1,lambda_2),N_bis,nu,3);
end

gamma_0=gamma;
display(['First rescaling, gamma = ',num2str(gamma)])

% Now we compute the other coefficients using Newton's method
k=0;
while error>10^-14 && k<100
    a_bis(10:end)=a_bis(10:end)-DF(a_bis,N_bis,sigma,s,Delta,zeta,epsilon,lambda_1,lambda_2)\F(a_bis,N_bis,sigma,s,Delta,zeta,epsilon,lambda_1,lambda_2);
    k=k+1;
    error=norm_multiD(F(a_bis,N_bis,sigma,s,Delta,zeta,epsilon,lambda_1,lambda_2),N_bis,nu,3);
end
if error>10^-14
    display('Newton did not converge')
    display(error)
end


%% FULL COMPUTATION OF a, INITIALIZED WITH a_bis, USING NEWTON'S METHOD
display('Full computation with Newton Method')
a=zeros(3*N*(N+1)/2,1);
M=min([N,N_bis]);
a(1:3*M*(M+1)/2,1)=a_bis(1:3*M*(M+1)/2,1);

k=0;
error=norm_multiD(F(a,N,sigma,s,Delta,zeta,epsilon,lambda_1,lambda_2),N,nu,3);
while error>10^-17 && k<10
    a(10:end)=a(10:end)-DF(a,N,sigma,s,Delta,zeta,epsilon,lambda_1,lambda_2)\F(a,N,sigma,s,Delta,zeta,epsilon,lambda_1,lambda_2);
    k=k+1;
    error=norm_multiD(F(a,N,sigma,s,Delta,zeta,epsilon,lambda_1,lambda_2),N,nu,3);
end
display(error)   



%% NOW WE FIND THE RESCALING (gamma,gamma) SUCH THAT THE l^1_{\nu}$ NORM OF $F(a)$ IS EQUAL TO eps_max
display('Rescaling with fixed ratio')

[F1,F2,F3]=coef_F(a,N,sigma,s,Delta,zeta,epsilon,lambda_1,lambda_2);
gamma=1;
val=defect(F1,F2,F3,N,nu,gamma,gamma);
relative_error=10^-2;
if val<eps_max
    while val<eps_max
        gamma=2*gamma;
        val=defect(F1,F2,F3,N,nu,gamma,gamma);
    end
    a_dicho=gamma/2;
    b_dicho=gamma;
    while abs(val-eps_max)/val>relative_error
       gamma=(a_dicho+b_dicho)/2;
       val=defect(F1,F2,F3,N,nu,gamma,gamma);
       if val<eps_max
           a_dicho=gamma;
       else
           b_dicho=gamma;
       end
    end
else
    while val>eps_max
        gamma=gamma/2;
        val=defect(F1,F2,F3,N,nu,gamma,gamma);
    end
    a_dicho=gamma;
    b_dicho=2*gamma;
    while abs(val-eps_max)/val>relative_error
       gamma=(a_dicho+b_dicho)/2;
       val=defect(F1,F2,F3,N,nu,gamma,gamma);
       if val<eps_max
           a_dicho=gamma;
       else
           b_dicho=gamma;
       end
    end
end
a=a.*gamma.^multiple(V_alpha_1(N),3).*gamma.^multiple(V_alpha_2(N),3);

display(gamma)

gamma_0=gamma_0*gamma;
gamma_1=gamma_0;
gamma_2=gamma_0*ratio;

