function [a,gamma_1,gamma_2]=compute_para_Lorenz_max(sigma,beta,rho,N,nu,eps_max)
%%% This function compute an approximate parameterization of degre N of 
%%% the stable mnifold at the origin for the Lorenz system (with parameters
%%% sigma, beta and rho), that maximize the surface area of the image while
%%% having a defect being less than eps_max. 
%%% Outputs: the parameterization is gaiven by a vector a. gamma_1 and
%%% gamma_2 are the rescaling applied to the first and second eigenvector 
%%% respectively.


p=[0;0;0];
lambda_1=-sigma/2-1/2-1/2*sqrt(sigma^2-2*sigma+1+4*sigma*rho);
lambda_2=-beta;
V1=[sigma/(lambda_1+sigma);1;0];
V2=[0;0;1];
display(lambda_1)
display(lambda_2)

test_real=1;

%% FIRST EASY NEWTON COMPUTATION WITH A FEW MODES
N_bis=10;
a_bis=zeros(3*N_bis*(N_bis+1)/2,1);
a_bis(1:3)=p;
a_bis(4:6)=V1;
a_bis(7:9)=V2;
error=norm_multiD(F(a_bis,N_bis,sigma,beta,rho,lambda_1,lambda_2),N,nu,3);

% Before trying Newton's method, we now ensure that the scale is small enough,
% if not we rescale the eigenvectors. If one of the Newton's method still does
% not converge, decrease scale_para.
gamma=1;
scale_para=1;
while error>scale_para
    gamma=gamma/2;
    a_bis(4:6)=gamma*V1;
    a_bis(7:9)=gamma*V2;
    error=norm_multiD(F(a_bis,N_bis,sigma,beta,rho,lambda_1,lambda_2),N,nu,3);
end

gamma_0=gamma;
display(['First rescaling, gamma = ',num2str(gamma)])

% Now we compute the other coefficients using Newton's method.
k=0;
while error>10^-14 && k<100
    a_bis(10:end)=a_bis(10:end)-DF(a_bis,N_bis,sigma,beta,rho,lambda_1,lambda_2)\F(a_bis,N_bis,sigma,beta,rho,lambda_1,lambda_2);
    k=k+1;
    error=norm_multiD(F(a_bis,N_bis,sigma,beta,rho,lambda_1,lambda_2),N,nu,3);
end
if error>10^-14
    display('Newton did not converge')
    display(error)  
end


%% FULL COMPUTATION OF a, INITIALIZED WITH a_bis, USING NEWTON'S METHOD
display('Full computation with Newton Method')
a=zeros(3*N*(N+1)/2,1);
M=min([N,N_bis]);
a(1:3*M*(M+1)/2,1)=a_bis(1:3*M*(M+1)/2,1);

k=0;
error=norm_multiD(F(a,N,sigma,beta,rho,lambda_1,lambda_2),N,nu,3);
% If scale_para was too small, it is possible that test is already really
% small and that you don't even enter in the Newton loop, in which case it
% is as if you still had only N_bis coefficients.
while error>10^-18 && k<5
    a(10:end)=a(10:end)-DF(a,N,sigma,beta,rho,lambda_1,lambda_2)\F(a,N,sigma,beta,rho,lambda_1,lambda_2);
    k=k+1;
    error=norm_multiD(F(a,N,sigma,beta,rho,lambda_1,lambda_2),N,nu,3);
end
display(error)


%% NOW WE FIND THE RESCALING (gamma,gamma) SUCH THAT THE l^1_{\nu}$ NORM OF $F(a)$ IS EQUAL TO eps_max
display('Uniform rescaling')

relative_error=10^-2;

[F1,F2,F3]=coef_F(a,N,sigma,beta,rho,lambda_1,lambda_2);
gamma=1;
val=defect(F1,F2,F3,N,nu,gamma,gamma);
if val<eps_max
    while val<eps_max
        gamma=2*gamma;
        val=defect(F1,F2,F3,N,nu,gamma,gamma);
    end
    a_dicho=gamma/2;
    b_dicho=gamma;
    while abs(val-eps_max)/val>relative_error
       gamma=(a_dicho+b_dicho)/2;
       val=defect(F1,F2,F3,N,nu,gamma,gamma);
       if val<eps_max
           a_dicho=gamma;
       else
           b_dicho=gamma;
       end
    end
else
    while val>eps_max
        gamma=gamma/2;
        val=defect(F1,F2,F3,N,nu,gamma,gamma);
    end
    a_dicho=gamma;
    b_dicho=2*gamma;
    while abs(val-eps_max)/val>relative_error
       gamma=(a_dicho+b_dicho)/2;
       val=defect(F1,F2,F3,N,nu,gamma,gamma);
       if val<eps_max
           a_dicho=gamma;
       else
           b_dicho=gamma;
       end
    end
end


a=a.*gamma.^multiple(V_alpha_1(N),3).*gamma.^multiple(V_alpha_2(N),3);

display(gamma)
gamma_0=gamma_0*gamma;

%% FINALLY, WE COMPUTE THE CURVE OF (gamma1,gamma2) ON WHICH  THE $l^1_{\nu}$ NORM OF $F(a)$ IS EQUAL TO eps_max,
%% AND FIND THE MAXIMUM OF THE SURFACE OF THE MANIFOLD ON THIS CURVE

display('Anisotropic rescaling')
format short

[F1,F2,F3]=coef_F(a,N,sigma,beta,rho,lambda_1,lambda_2);

Tab_gamma=[];
Tab_surf=[];
gamma2=1;
relative_error2=10^-4;
% If the surface is monotonone for this range of gamma1, you need to change
% it in order to get the maximal surface
for gamma1=1.5:.1:2
    display(gamma1)
    val=defect(F1,F2,F3,N,nu,gamma1,gamma2);
    if val<eps_max
        while val<eps_max
            gamma2=2*gamma2;
            val=defect(F1,F2,F3,N,nu,gamma1,gamma2);
        end
        a_dicho=gamma2/2;
        b_dicho=gamma2;
        while abs(val-eps_max)/val>relative_error2
           gamma2=(a_dicho+b_dicho)/2;
           val=defect(F1,F2,F3,N,nu,gamma1,gamma2);
           if val<eps_max
               a_dicho=gamma2;
           else
               b_dicho=gamma2;
           end
        end
    else
        while val>eps_max
            gamma2=gamma2/2;
            val=defect(F1,F2,F3,N,nu,gamma1,gamma2);
        end
        a_dicho=gamma2;
        b_dicho=2*gamma2;
        while abs(val-eps_max)/val>relative_error2
           gamma2=(a_dicho+b_dicho)/2;
           val=defect(F1,F2,F3,N,nu,gamma1,gamma2);
           if val<eps_max
               a_dicho=gamma2;
           else
               b_dicho=gamma2;
           end
        end
    end
    Tab_gamma=[Tab_gamma [gamma1;gamma2]];
    Tab_surf=[Tab_surf surface_value(a.*gamma1.^multiple(V_alpha_1(N),3).*gamma2.^multiple(V_alpha_2(N),3),N,nu,test_real)];
end


[~,index]=max(Tab_surf);
gamma1=Tab_gamma(1,index);
gamma2=Tab_gamma(2,index);

a=a.*gamma1.^multiple(V_alpha_1(N),3).*gamma2.^multiple(V_alpha_2(N),3);

gamma_1=gamma1*gamma_0;
gamma_2=gamma2*gamma_0;

%Plot the value of gamma2 in function of gamma1 
figure
plot(Tab_gamma(1,:),Tab_gamma(2,:),'.')

%Plot the value of the surface area in function of gamma1
figure
plot(Tab_gamma(1,:),Tab_surf,'.b')


