%% THIS SCRIPT COMPUTES AND DISPLAYS THE STABLE MANIFOLD OF THE ORIGIN AND 
%% THE UNSTABLE MANIFOLD FOR BOTH EYES. THE COMPUTATION OF EACH MANIFOLD IS 
%% DONE TO MAXIMIZE THE SURFACE AREA WHILE HAVING A DEFECT LESS THAN 
%% eps_max.

sigma = 10;
beta = 8/3;
rho = 28;

N = 30;
nu = 1;
eps_max = 10^-5;

%Compute the Stable Manifold of the Origin:
point=0;
[a0,test_real0]=compute_para_Lorenz(sigma,beta,rho,N,nu,eps_max,point);

%Compute the Unstable Manifold of eye 1:
point=1;
[a1,test_real1]=compute_para_Lorenz(sigma,beta,rho,N,nu,eps_max,point);

%Compute the Unstable Manifold of eye 2:
point=2;
[a2,test_real2]=compute_para_Lorenz(sigma,beta,rho,N,nu,eps_max,point);


mesh_size=20;
num_angles=30;
num_rad = 10;

%Triangulate the Local Stable Manifold at Origin:
[TRI0, W0] = triangulate(a0,N,nu,mesh_size,test_real0);

%Triangulate the Local Unstable Manifold at Eye 1:
[TRI1, W1] = triangulate_complex(a1,N,nu,num_angles,num_rad,test_real1);

%Triangulate the Local Unstable Manifold at Eye 2:
[TRI2, W2] = triangulate_complex(a2,N,nu,num_angles,num_rad,test_real2);



%Compute the attractor:
tspan = 45;
IC = [1,1,1];
options=odeset('RelTol',1e-13,'AbsTol',1e-13);
[Time1, testOrbit1] = ode45(...
    'lorenzField', tspan, IC, options, flag, sigma, rho, beta, 1);

IC = [-1,-1,-1];
options=odeset('RelTol',1e-13,'AbsTol',1e-13);
[Time2, testOrbit2] = ode45(...
    'lorenzField', tspan, IC, options, flag, sigma, rho, beta, 1);


%Plotting:
figure
hold on
t0 = trisurf(TRI0, W0(:,1),  W0(:,2), W0(:,3));
set(t0, 'FaceColor', 'red');
t1 = trisurf(TRI1, W1(:,1),  W1(:,2), W1(:,3));
set(t1, 'FaceColor', 'blue');
t2 = trisurf(TRI2, W2(:,1),  W2(:,2), W2(:,3));
set(t2, 'FaceColor', 'blue');

shading flat;                   
% other options are shading face / shading faceted

plot3(testOrbit1(60000:end,1), testOrbit1(60000:end,2), ...
    testOrbit1(60000:end,3), 'color', [0.1 0.8 0.1])                           
plot3(testOrbit2(60000:end,1), testOrbit2(60000:end,2), ...
    testOrbit2(60000:end,3), 'color', [0.1 0.8 0.1])                           
hold off  


 camlight headlight
 