function val=surface_value(a,N,nu,test_real)

% This function computes the surface area of the manifold defined by the
% parameterization a on the domain of size nu, using the formula
% $\int || \partial_1 f ^ \partial_2 f \\$.
% The parameter isreal specifies if the parameterization a is real (isreal=1)
% or not (isreal=0).


if test_real
    pv=prod_vect(partial_derivative(a,N,1),partial_derivative(a,N,2),N);
    [M1,M2,M3]=vect2mat(pv,N);
    f=@(x,y) eval_para_vectorialized(x,y,N,M1,M2,M3);
    val=integral2(f,-nu,nu,-nu,nu);
else
    pv=prod_vect(partial_derivative(a,N,1)+partial_derivative(a,N,2),1i*(partial_derivative(a,N,1)-partial_derivative(a,N,2)),N);
    [M1,M2,M3]=vect2mat(pv,N);
    g=@(r,phi) r.*eval_para_vectorialized(r.*exp(1i*phi),r.*exp(-1i*phi),N,M1,M2,M3);
    val=integral2(g,0,nu,0,2*pi);
end

