function [TRI, W] = triangulate(a,N,nu,mesh_size,test_real)

%==========================================================================
%Construction of the mesh:
%==========================================================================

x_points = linspace(-nu, nu, mesh_size);
y_points = linspace(-nu, nu, mesh_size);

m_int = mesh_size; 
n_int = mesh_size; 

totalPossiblePoints = m_int*n_int;
D_parms = zeros(totalPossiblePoints, 2);

thisIndex = 1;

for i = 1:m_int
    for j = 1:n_int 
       D_parms(thisIndex, 1:2) = [x_points(i), y_points(j)];    
       thisIndex = thisIndex + 1;
    end
end    

D_parms = unique(D_parms, 'rows');

%==========================================================================
%Triangulation of the parameter domain:
%==========================================================================

x = D_parms(:,1);
y = D_parms(:,2);

TRI = delaunay(x, y);

figure
hold on
triplot(TRI, D_parms(:,1),  D_parms(:,2), 'red');

%==========================================================================
%Evaluation of the paramterization:
%==========================================================================

numPoints = size(D_parms, 1);
W = zeros(numPoints, 3);
[M1,M2,M3]=vect2mat(a,N);

for k = 1:numPoints
    W(k, 1:3) = eval_parameterization(...
        D_parms(k, 1), D_parms(k, 2),M1,M2,M3,N,test_real)';
end